var express = require('express'),
    app = module.exports = express.createServer(),
    HoganTemplateRenderer = require('hogan-template-compiler'),

    templateDirectory = __dirname + "/views",
    layoutTemplatePath = templateDirectory + "/layout.mustache",
    hoganTemplateRenderer = HoganTemplateRenderer({
        partialsDirectory: templateDirectory + "/partials"
    });

/**
 * An Express compatible template compiler.
 */
var hoganCompiler = {
    compile: function(source, options) {
        var template;
        if (options.filename === layoutTemplatePath) {
            template = hoganTemplateRenderer.compileTemplateFile(layoutTemplatePath);
        } else {
            template = hoganTemplateRenderer.getTemplate(options.filename);
        }

        return function(locals) {
            return template.render(locals, hoganTemplateRenderer.getPartials());
        };
    }
};

app.configure(function() {
    // Register the hogan renderer with the mustache file type.
    app.register('.mustache', hoganCompiler);

    // Set the default template extension to .mustache
    app.set('view engine', 'mustache');

    app.set('views', templateDirectory);
    app.use(express.bodyParser());
    app.use(express.methodOverride());
    app.use(app.router);
    app.use(express.static(__dirname + '/public'));
});

app.configure('development', function(){
    /**
     * Re-read the templates for each request in development mode.
     */
    app.use(function(req, res, next) {
        hoganTemplateRenderer.read();
        next();
    });

    app.use(express.errorHandler({ dumpExceptions: true, showStack: true }));
});

app.configure('production', function() {
  app.use(express.errorHandler());
});

/**
 * Request handler for pre-compiled hogan.js templates.
 *
 * This function uses a hogan template of it's own which renders
 * calls to Hogan.Template. See views/sharedTemplates.mustache.
 */
app.get("/templates.js",  function(req, res) {
    res.contentType("templates.js");
    res.send(hoganTemplateRenderer.getSharedTemplates());
});

/**
 * Request handler for the homepage.
 *
 * Renders a hogan template on the server side which contains a form 
 * which will update the article section on the client using the
 * pre-compiled template.
 */
app.get("/", function(req, res) {
    res.render("layout", {
        headline: "This is a server-side rendered headline",
        bodyText: "This is some bodytext"
    });
});

app.listen(3000);
console.log("Express server listening on port %d", app.address().port);